@@ -103,24 +103,23 @@ class Granule:
             df['lat'] = self.lat.flatten()
             df['lon'] = self.lon.flatten()
         if self.sids is not None:
-            # Converting to nullable series
+            # Converting to nullable series (capitalized "I" in Int64)
             sids = pandas.Series(pandas.array(self.sids.flatten(), dtype='Int64'))
-            df['sids'] = sids #self.sids.flatten()
+            df['sids'] = sids
         if self.ts_start is not None and self.ts_end is not None:
             df['ts_start'] = self.ts_start
             df['ts_end'] = self.ts_end
         if xy:
             if self.lat is not None:
-                # Todo: those should probably be int16s, not int64s
                 indices = numpy.indices(self.lat.shape, dtype='uint16')
             elif self.sids is not None:
-                indices = numpy.indices(self.sids.shape)
+                indices = numpy.indices(self.sids.shape, dtype='uint16')
             else:
                 ds = list(self.data.values())[0]
-                indices = numpy.indices(ds.shape)
-            # Argh. not always True; sometimes x i the second index.
-            df['x'] = indices[0].flatten()
-            df['y'] = indices[1].flatten()
+                indices = numpy.indices(ds.shape, dtype='uint16')
+            # Careful indices() gives first the row (y), then the column (x) indices
+            df['x'] = indices[1].flatten()
+            df['y'] = indices[0].flatten()
         for key in self.data.keys():
             dtype = self.data[key].dtype
 
